@@ -88,6 +88,24 @@ def get_additional_type_values(element, value_type, return_elements=False):
   return elements
 
 
+def get_language_to_text_map(element):
+  """Return a map of languages to text in an InternationalizedText element."""
+  language_map = {}
+  if element is None:
+    return language_map
+  intl_strings = element.findall("Text")
+
+  for intl_string in intl_strings:
+    text = intl_string.text
+    if text is None or not text:
+      continue
+    language = intl_string.get("language")
+    if language is None or not language:
+      continue
+    language_map[language] = text
+  return language_map
+
+
 class Schema(base.TreeRule):
   """Checks if election file validates against the provided schema."""
 
@@ -1901,6 +1919,38 @@ class ValidJurisdictionID(base.ValidReferenceRule):
     return {elem.get("objectId") for elem in gp_unit_elements}
 
 
+class OfficesHaveValidOfficeLevel(base.BaseRule):
+  """Each office must have a valid office-level."""
+
+  def elements(self):
+    return ["Office"]
+
+  def check(self, element):
+    valid_office_level_values = {
+        "Country", "Municipality", "Neighbourhood", "District", "Region",
+        "International", "Ward", "Administrative Area 1",
+        "Administrative Area 2"
+    }
+    office_level_values = [
+        ol_id.strip()
+        for ol_id in get_external_id_values(element, "office-level")
+        if ol_id.strip()
+    ]
+    if not office_level_values:
+      raise loggers.ElectionError(
+          ("Office {} is missing an office-level.".format(
+              element.get("objectId", ""))))
+    if len(office_level_values) > 1:
+      raise loggers.ElectionError(
+          ("Office {} has more than one office-level.".format(
+              element.get("objectId", ""))))
+    office_level_value = office_level_values[0]
+    if office_level_value not in valid_office_level_values:
+      raise loggers.ElectionError(
+          ("Office {} has invalid office-level {}.".format(
+              element.get("objectId", ""), office_level_value)))
+
+
 class ElectionStartDates(base.DateRule):
   """Election elements should contain valid start dates.
 
@@ -1979,6 +2029,135 @@ class GpUnitsHaveInternationalizedName(base.BaseRule):
                element.get("object_id", ""), "\n".join(missing_names))))
 
 
+class FullTextMaxLength(base.BaseRule):
+  """FullText field should not be longer than MAX_LENGTH."""
+
+  MAX_LENGTH = 30000  # about 8-10 pages of text, 4500-5000 words
+
+  def elements(self):
+    return ["FullText"]
+
+  def check(self, element):
+    intl_text_list = element.findall("Text")
+    for intl_text in intl_text_list:
+      if len(intl_text.text) > self.MAX_LENGTH:
+        raise loggers.ElectionWarning(
+            "FullText is longer than %s characters.  Please remove and "
+            "include a link to the full text via InfoUri with Annotation "
+            "'fulltext'." % (self.MAX_LENGTH))
+
+
+class FullTextOrBallotText(base.BaseRule):
+  """Warn if BallotText is missing and FullText is short."""
+
+  SUGGESTION_CUTOFF_LENGTH = 2500  # about 3 paragraphs, 250-300 words
+
+  def elements(self):
+    return ["BallotMeasureContest"]
+
+  def check(self, element):
+    full_text_map = get_language_to_text_map(element.find("FullText"))
+    if not full_text_map:
+      return
+
+    ballot_text_map = get_language_to_text_map(element.find("BallotText"))
+    for language, full_text_string in full_text_map.items():
+      if language not in ballot_text_map.keys(
+      ) and len(full_text_string) < self.SUGGESTION_CUTOFF_LENGTH:
+        raise loggers.ElectionWarning(
+            "Line: %d. Language: %s.  BallotText is missing but FullText is "
+            "present for the same language. Please confirm that FullText "
+            "contains only supplementary text and not text on the ballot "
+            "itself." % (element.sourceline, language))
+
+
+class BallotTitle(base.BaseRule):
+  """BallotTitle must exist and should usually be shorter than BallotText."""
+
+  def elements(self):
+    return ["BallotMeasureContest"]
+
+  def check(self, element):
+    ballot_title_map = get_language_to_text_map(element.find("BallotTitle"))
+    if not ballot_title_map:
+      raise loggers.ElectionError(
+          "Line %d. BallotMeasureContest is missing BallotTitle." %
+          (element.sourceline))
+
+    ballot_text_map = get_language_to_text_map(element.find("BallotText"))
+    if not ballot_text_map:
+      raise loggers.ElectionWarning(
+          "Line %d. BallotText is missing. Please confirm that the ballot "
+          " text/question is not in BallotTitle." % (element.sourceline))
+
+    for language, ballot_title_string in ballot_title_map.items():
+      if language not in ballot_text_map.keys() or len(
+          ballot_text_map[language]) < len(ballot_title_string):
+        raise loggers.ElectionWarning(
+            "Line: %d. Language: %s. BallotText is missing or shorter than "
+            "BallotTitle. Please confirm that the ballot text/question is not "
+            "in BallotTitle." % (element.sourceline, language))
+
+
+class ImproperCandidateContest(base.TreeRule):
+  """Detect CandidateContest elements that should be a BallotMeasureContest."""
+
+  _BALLOT_SELECTION_OPTIONS = frozenset({"yes", "no", "for", "against"})
+
+  def _gather_contest_candidates(self, contest):
+    """Return candidate ids for given contest element."""
+    candidate_ids = []
+    cand_id_elems = contest.findall("BallotSelection//CandidateIds")
+    for cand_id_elem in cand_id_elems:
+      for cand_id in cand_id_elem.text.split():
+        candidate_ids.append(cand_id)
+    return candidate_ids
+
+  def _gather_invalid_candidates(self):
+    """Return candidate ids that appear to be BallotMeasureSelections."""
+    invalid_candidates = []
+    candidates = self.get_elements_by_class(
+        self.election_tree, "CandidateCollection//Candidate")
+    for candidate in candidates:
+      ballot_name = candidate.find(".//BallotName/Text[@language='en']")
+      if ballot_name is not None:
+        if ballot_name.text.lower() in self._BALLOT_SELECTION_OPTIONS:
+          invalid_candidates.append(candidate.get("objectId"))
+    return invalid_candidates
+
+  def check(self):
+    candidate_contest_mapping = {}
+    candidate_contests = self.get_elements_by_class(
+        self.election_tree, "CandidateContest")
+    for cc in candidate_contests:
+      cand_ids = self._gather_contest_candidates(cc)
+      contest_id = cc.get("objectId")
+      candidate_contest_mapping[contest_id] = cand_ids
+
+    invalid_candidates = self._gather_invalid_candidates()
+
+    error_log = []
+    for contest_id, cand_ids in candidate_contest_mapping.items():
+      flagged_candidates = []
+      for cand_id in cand_ids:
+        if cand_id in invalid_candidates:
+          flagged_candidates.append(cand_id)
+      if flagged_candidates:
+        warning_message = ("Candidates {} should be BallotMeasureSelection "
+                           "elements. Similarly, Contest {} should be changed "
+                           "to a BallotMeasureContest instead of a "
+                           "CandidateContest.").format(
+                               ", ".join(flagged_candidates), contest_id)
+        error_log.append(loggers.ErrorLogEntry(None, warning_message))
+
+    if invalid_candidates:
+      warning_message = ("There are CandidateContests that appear to be "
+                         "BallotMeasureContests based on the "
+                         "BallotName values.")
+      raise loggers.ElectionWarning("There are misformatted contests.",
+                                    error_log)
+
+
 class RuleSet(enum.Enum):
   """Names for sets of rules used to validate a particular feed type."""
   ELECTION = 1
@@ -2013,6 +2192,7 @@ COMMON_RULES = (
     GpUnitsCyclesRefsValidation,
     ValidJurisdictionID,
     OfficesHaveJurisdictionID,
+    OfficesHaveValidOfficeLevel,
     ValidStableID,
     PersonHasUniqueFullName,
     PersonsMissingPartyData,
@@ -2041,6 +2221,10 @@ ELECTION_RULES = COMMON_RULES + (
     DuplicatedPartyName,
     DuplicatedPartyAbbreviation,
     MissingPartyNameTranslation,
+    FullTextMaxLength,
+    FullTextOrBallotText,
+    BallotTitle,
+    ImproperCandidateContest,
 )
 
 OFFICEHOLDER_RULES = COMMON_RULES + (
